# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2011
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
###########################################################################
"""
Primitives pour la gestion des ACL
"""
from os.path import isdir
from pyeole.process import system_out #, system_code

def get_acl(path):
    """
    renvoie les ACL posées sur un répertoire
    """
    cmd = ['getfacl', '--access', '--absolute-names', path]
    ret, acl, err = system_out(cmd)
    if ret:
        raise Exception, err
    return acl

def set_owner(path, user, recursive=True):
    """
    définit le propriétaire d'un fichier (chown)
    """
    if recursive:
        cmd = ['/bin/chown', '-R', user, path]
    else:
        cmd = ['/bin/chown', user, path]
    system_out(cmd)

def set_group(path, group, recursive=True):
    """
    définit le groupe propriétaire d'un fichier (chgrp)
    """
    if recursive:
        cmd = ['/bin/chgrp', '-R', group, path]
    else:
        cmd = ['/bin/chgrp', group, path]
    system_out(cmd)

def set_user_acl(path, user, mode='rwx', recursive=True):
    """
    ajoute une ACL utilisateur sur un répertoire/fichier
    """
    rights = 'u:%s:%s' % (user, mode)
    set_acl(path, rights, recursive=recursive)

def set_group_acl(path, group, mode='rwx', recursive=True):
    """
    ajoute une ACL groupe sur un répertoire/fichier
    """
    rights = 'g:%s:%s' % (group, mode)
    set_acl(path, rights, recursive=recursive)

def set_acl(path, rights, recursive=True):
    """
    ajoute une ACL sur un répertoire/fichier
    """
    if recursive:
        opts = '-PRm'
    else:
        opts = '-Pm'
    cmd = ['setfacl', opts, rights, path]
    system_out(cmd)

def set_default_acl(path, rights, recursive=True):
    """
    ajoute une ACL par défaut sur un répertoire
    """
    if not isdir(path):
        return
    if recursive:
        opts = '-PRdm'
    else:
        opts = '-Pdm'
    cmd = ['setfacl', opts, rights, path]
    system_out(cmd)

def clear_acl(path, recursive=True):
    """
    supprime toutes les ACL posées sur un répertoire/fichier
    """
    # suppression des droits group + other
    set_acl(path, 'g::---,o::---', recursive=recursive)
    if recursive:
        opts = '-PRbk'
    else:
        opts = '-Pbk'
    system_out(['setfacl', opts, path])

def clear_default_acl(path, recursive=True):
    """
    supprime toutes les ACL par défaut posées sur un répertoire/fichier
    """
    if recursive:
        opts = '-PRk'
    else:
        opts = '-Pk'
    system_out(['setfacl', opts, path])

def copy_default_acl(path, recursive=True):
    """
    ajoute les ACL par défaut sur un répertoire/fichier
    (copie les droits existants)
    """
    # suppression des éventuelles ACL par défaut
    clear_default_acl(path, recursive=recursive)
    # récupération des ACL courantes
    rights = get_acl(path)
    # mise à jour des acl par défaut
    if recursive:
        opts = '-RPd'
    else:
        opts = '-Pd'
    cmd = ['setfacl', opts, '-M-', path]
    ret, acl, err = system_out(cmd, stdin=rights)
    if ret:
        raise Exception, err

