# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2010
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill
# eole@ac-dijon.fr
###########################################################################
"""
Gestion et application des modèles de partage Samba
"""
from glob import glob
from os import listdir, makedirs
from os.path import join, isdir, isfile
from creole.client import CreoleClient
from pyeole.service import service_code
from ConfigParser import ConfigParser

DICO = None
CONFIG_FILE = '/var/lib/eole/config/ead.cfg'

if not isfile(CONFIG_FILE):
    print("Fichier de configuration non existant")
    exit(0)

cfg = ConfigParser(allow_no_value=True)
cfg.read(CONFIG_FILE)

FICHIER_PATH = cfg.get('eole', 'container_path_fichier')
if FICHIER_PATH == '':
    FICHIER_PATH = '/'
SAMBA_PATH = join(FICHIER_PATH, 'etc/samba')
INCLUDE_PATH = join(SAMBA_PATH, 'conf.d')

def get_share_models():
    """
    retourne la liste des modèles de partage disponibles
    """
    templates = []
    files = listdir('/usr/share/eole/fichier/models/')
    for fil in files:
        if fil.endswith('.tmpl'):
            templates.append(fil[0:-5])
    if 'standard' not in templates:
        # il en faut au moins un
        templates.append('standard')
    templates.sort()
    return templates

def add_smb_include():
    """
    ajoute les inclusions des fichiers présents dans /etc/samba/conf.d
    """
    smb_conf = join(SAMBA_PATH, 'smb.conf')
    content = file(smb_conf).read()

    # inclusion des fichiers /etc/samba/conf.d/*.conf
    smb_include = []
    for include_file in glob(join(INCLUDE_PATH, '*.conf')):
        if FICHIER_PATH != '/':
            # short path in container
            include_file = include_file.replace(FICHIER_PATH, '')
        if include_file in content:
            # fichier déjà inclus
            continue
        smb_include.append('include = %s\n' % include_file)
    if smb_include != []:
        smb_include.append('\n')
        fic = open(smb_conf, 'a')
        fic.writelines(smb_include)
        fic.close()

def synchronize(shares, restart=True):
    """
    Ajout des partages ldap dans le fichier smb.conf
    @shares : liste de dictionnaires contenants les clés :
        name, path, group, desc, model
    """
    eole_conf = join(INCLUDE_PATH, 'eole.conf')
    templates = {}
    for tmpl in get_share_models():
        try:
            fic = open('/usr/share/eole/fichier/models/%s.tmpl' % tmpl)
            templates[tmpl] = fic.read()
            fic.close()
        except:
            print "Erreur de lecture du template %s" % tmpl

    default = """[%(name)s]
        inherit permissions = no
        inherit acls = yes
        read only = no
        preserve case = no
        create mask = 644
        directory mask = 755
        Write list = @%(group)s
        Valid Users = @%(group)s @"Domain Admins"
        path = %(path)s
"""
    #result = ldapsearch(share_filters)
    #for share in result:
    #    dico_share = {}
    #    dico_share['name'] = share[1]['sambaShareName'][0]
    #    dico_share['path'] = share[1]['sambaFilePath'][0]
    #    dico_share['group'] = share[1]['sambaShareGroup'][0]
    #    dico_share['desc'] = share[1]['description'][0]
    smb = []
    for dico_share in shares:
        # Merge DICO to use eole variables
        global DICO
        if DICO is None:
            DICO = CreoleClient().get_creole()
        for key in DICO:
            if not dico_share.has_key(key):
                dico_share[key] = DICO[key]
        model = dico_share.get('model', '')
        if model == '':
            model = 'standard'
            print "modèle non défini pour le partage %s" % dico_share['name']

        ## FIXME : création du répertoire si manquant
        #if not dico_share['path'].startswith('%') and is_group(dico_share['group']):
        #    if not isdir(dico_share['path']) or not verif_acl(dico_share['path'], dico_share['group']):
        #        system("mkdir -p %(path)s;chmod 770 %(path)s" % dico_share)
        #        if share[1].has_key('sambaShareOptions'):
        #            if 'sticky' in share[1]['sambaShareOptions']:
        #                system ("chgrp %(group)s %(path)s;chmod 1770 %(path)s;" % dico_share)
        #        set_acl(dico_share['path'], 'root', dico_share['group'])
        ## génération du template pour samba
        try:
            smb.append(templates[model] % dico_share)
        except KeyError:
            print "Le modèle de partage %s n'existe pas" % model
            smb.append(default % dico_share)
        except:
            print "Erreur lors de la templatisation du modèle %s" % model
            smb.append(default % dico_share)
        smb.append('\n')
    # écriture du fichier final
    if not isdir(INCLUDE_PATH):
        makedirs(INCLUDE_PATH)
    fic = open(join(eole_conf), 'w')
    fic.writelines(smb)
    fic.close()

    # vérification des inclusions Samba
    add_smb_include()

    if restart:
        service_code('smbd', 'restart', 'fichier')
