# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2010
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
###########################################################################
"""
Gestion des mots de passe des utilisateurs
"""
import re
from pyeole.process import system_out

def mod_password(login, user_passwd, container='fichier'):
    """
    changement d'un mot de passe

    :param container: redéfinir à None si on est déjà dans fichier
    """
    user_passwd = user_passwd.replace('"', '\\"')
    cmd = ['/usr/sbin/smbldap-passwd', login]
    stdin = """%s\n%s""" % (user_passwd, user_passwd)
    ret = system_out(cmd, stdin=stdin, container=container)
    if ret[0] != 0:
        raise Exception(' '.join(ret[1:]))
    return True

def password_must_change(login, container='fichier'):
    """
    demande de changement de mot de passe forcée

    :param container: redéfinir à None si on est déjà dans fichier
    """
    cmd = ['/usr/bin/net', 'sam', 'set', 'pwdmustchangenow', login, 'yes']
    ret = system_out(cmd, container=container)
    if ret[0] != 0:
        raise Exception(' '.join(ret[1:]))
    return True

def check_nb_min_classes(user_passwd, nb_min_classes):
    """
    Vérifie que le mot de passe contient au moins
    le nombre de classes de caractères requis
    """
    nb_classes = 0
    regs = (r'[a-z]', r'[A-Z]', r'[0-9]', r'[^a-zA-Z0-9]')
    for regexp in regs:
        if re.search(regexp, user_passwd) is not None:
            nb_classes += 1
            if nb_classes >= nb_min_classes:
                return True
    return False

def password_is_valid(username, user_passwd, nb_min_classes, min_size):
    """
    Vérifie que le mot de passe est valide selon les criètres
    suivants :
      - nombre de classes de cractères
      - ne contiens pas le nom d'utilisateur
    """

    if len(user_passwd) < min_size:
        return False

    if check_nb_min_classes(user_passwd, nb_min_classes):
        if user_passwd.find(username) != -1:
            return False
    else:
        return False

    return True
